@@ -15,6 +15,8 @@ class TaskContext(object):
     :param egress_type_name: egress type name to use when calling send_egress_message()
     :param optional serialiser: serialiser to use (will use DefaultSerialiser if not set)
     """
+    __slots__ = ('_context', '_egress_type_name', '_serialiser', 'storage', 'task_state', 'pipeline_state', 'pipeline_state_size', '_task_meta', '_task_name', '_task_uid')
+
     def __init__(self, context: Context, egress_type_name: str, serialiser=None):
         self._context = context
         self._egress_type_name = egress_type_name
@@ -90,6 +92,14 @@ class TaskContext(object):
         """
         return self._task_meta.get('pipeline_id', None)
 
+    def get_pipeline_address(self):
+        """
+        Address of the pipeline if this task is called as part of a pipeline else None
+
+        :return: pipeline ID
+        """
+        return self._task_meta.get('pipeline_address', None)
+
     def get_parent_task_id(self):
         """
         ID of the parent task if this task has one else None
@@ -114,6 +124,23 @@ class TaskContext(object):
         """
         return f'{self._context.address.typename}'
 
+    def get_namespace(self):
+        """
+        Own namespace in the form of context.address.typename
+
+        :return: address
+        """
+        return f'{self._context.address.namespace}'
+
+    def get_worker_name(self):
+        """
+        Own worker_name in the form of context.address.typename
+
+        :return: address
+        """
+        return f'{self._context.address.name}'
+
+
     def get_task_id(self):
         """
         Own task Id in the form of context.address.id
@@ -138,17 +165,6 @@ class TaskContext(object):
         """
         return None if self._context.caller.id == "" else self._context.caller.id
 
-    def get_caller(self) -> Address:
-        """
-        Returns the caller of the task if there is one or None otherwise
-
-        :return: Address
-        """
-        if self._context.caller.id == "":
-            return None
-        else:
-            return Address(namespace=self._context.caller.namespace, type=self._context.caller.name, id=self._context.caller.id)
-
     def set_state(self, obj):
         self.task_state.internal_state.CopyFrom(pack_any(self._serialiser.to_proto(obj)))
 
@@ -168,7 +184,7 @@ class TaskContext(object):
         """
         return f'{address.namespace}/{address.type}', address.id
 
-    def send_message_after(self, delay: timedelta, destination, target_id, value):
+    def send_message_after(self, delay: timedelta, destination, target_id, value, cancellation_token: str = ""):
         """
         Sends a message to another Flink Task worker after some delay
 
@@ -176,23 +192,38 @@ class TaskContext(object):
         :param destination: the destination to send the message to (e.g. example/worker)
         :param target_id: the target Id
         :param value: the message to send
+        :param optional cancellation_token: a cancellation token to associate with this message
         """
         value_type = flink_value_type_for(value)
         message = message_builder(target_typename=destination, target_id=target_id, value=value, value_type=value_type)
-        self._context.send_after(delay, message)
+        self._context.send_after(delay, message, cancellation_token)
 
-    def send_message(self, destination, target_id, value):
+    def send_message(self, destination, target_id, value, delay: timedelta=None, cancellation_token: str = ""):
         """
         Sends a message to another Flink Task worker
 
         :param destination: the destination to send the message to (e.g. example/worker)
         :param target_id: the target Id
         :param value: the message to send
+        :param optional delay: the delay (if any then same behaviour send_message_after())
+        :param optional cancellation_token: a cancellation token to associate with this message
         """
+
+        if delay:
+            return self.send_message_after(delay, destination, target_id, value, cancellation_token)
+
         value_type = flink_value_type_for(value)
         message = message_builder(target_typename=destination, target_id=target_id, value=value, value_type=value_type)
         self._context.send(message)
 
+    def cancel_message(self, cancellation_token: str):
+        """
+        Attempts to cancel a message associated with this cancellation token
+
+        :param cancellation_token: the cancellation token
+        """
+        self._context.cancel_delayed_message(cancellation_token)
+
     def send_egress_message(self, topic, value):
         """
         Sends a message to an egress topic
