@@ -1,8 +1,7 @@
-from ast import If
-from statefun_tasks.types import Task, Group, RetryPolicy
+from statefun_tasks.types import Task, Group, RetryPolicy, ProtobufSerialisable
 from statefun_tasks.utils import _gen_id, _is_tuple
 from statefun_tasks.pipeline import _Pipeline
-from statefun_tasks.messages_pb2 import TaskEntry, TaskRequest, Pipeline
+from statefun_tasks.messages_pb2 import TaskRequest, Pipeline
 
 from typing import Iterable
 
@@ -11,7 +10,7 @@ def in_parallel(entries: list, max_parallelism=None):
     return PipelineBuilder().append_group(entries, max_parallelism)
 
 
-class PipelineBuilder(object):
+class PipelineBuilder(ProtobufSerialisable):
     """
     Builder class for creating pipelines of Flink Tasks
 
@@ -33,6 +32,16 @@ class PipelineBuilder(object):
     def id(self, value):
         self._builder_id = value
 
+    def append(self, task: Task) -> 'PipelineBuilder':
+        """
+        Appends a single task onto this pipeline
+
+        :param task: the task to append
+        :return: the builder
+        """
+        self._pipeline.append(task)
+        return self
+
     def append_to(self, other: 'PipelineBuilder') -> 'PipelineBuilder':
         """
         Appends tasks from another pipeline builder into this one
@@ -224,13 +233,13 @@ class PipelineBuilder(object):
         """
         errors = []
 
-        all_tasks = [task for task in self._pipeline if isinstance(task, Task)]
-        all_groups = [group for group in self._pipeline if isinstance(group, Group)]
+        all_tasks = self._get_tasks()
+        all_groups = self._get_groups()
 
         task_uids = [task.uid for task in all_tasks]
         if len(task_uids) != len(set(task_uids)):
             errors.append('Task uids must be unique')
-
+        
         group_ids = [group.group_id for group in all_groups]
         if len(group_ids) != len(set(group_ids)):
             errors.append('Group ids must be unique')
@@ -280,15 +289,7 @@ class PipelineBuilder(object):
 
         :return: list of tuples of namespace, worker name, id for each task
         """
-        def yield_tasks(entry):
-            for task_or_group in entry:
-                if isinstance(task_or_group, Group):
-                    for group_entry in task_or_group:
-                        yield from yield_tasks(group_entry)
-                else:
-                    yield task_or_group.namespace, task_or_group.worker_name, task_or_group.id
-        
-        return list(yield_tasks(self))
+        return [(t.namespace, t.worker_name, t.id) for t in self._get_tasks()]
 
     def is_empty(self):
         """
@@ -298,6 +299,27 @@ class PipelineBuilder(object):
         """
         return not any(self.get_tasks())
 
+    def _get_tasks(self) -> list:
+        def yield_tasks(entry):
+            for task_or_group in entry:
+                if isinstance(task_or_group, Group):
+                    for group_entry in task_or_group:
+                        yield from yield_tasks(group_entry)
+                else:
+                    yield task_or_group
+        
+        return list(yield_tasks(self))
+
+    def _get_groups(self) -> list:
+        def yield_groups(entry):
+            for task_or_group in entry:
+                if isinstance(task_or_group, Group):
+                    yield task_or_group
+                    for group_entry in task_or_group:
+                        yield from yield_groups(group_entry)
+        
+        return list(yield_groups(self))
+
     def __iter__(self):
         return self._pipeline.__iter__()
 
