@@ -13,17 +13,15 @@ from statefun_tasks.pipeline import _Pipeline
 from statefun_tasks.tasks import FlinkTask
 from statefun_tasks.task_impl.handlers import TaskRequestHandler, TaskResponseHandler, TaskActionHandler, ChildPipelineHandler
 from statefun_tasks.events import EventHandlers
+from statefun_tasks.builtin_tasks import run_pipeline
 
 from statefun import ValueSpec, Context, Message
+from datetime import timedelta
 import logging
 
 _log = logging.getLogger('FlinkTasks')
 
 
-def _run_pipeline(pipeline_proto):
-    return PipelineBuilder.from_proto(pipeline_proto).validate()
-
-
 class FlinkTasks(object):
     """
     Flink Tasks implementation
@@ -54,7 +52,7 @@ class FlinkTasks(object):
             ValueSpec(name="pipeline_state", type=PIPELINE_STATE_TYPE),
         ]
 
-        self.register_builtin('run_pipeline', _run_pipeline)
+        self.register_builtin('run_pipeline', run_pipeline)
 
         self._handlers = [
             TaskRequestHandler(),
@@ -267,10 +265,12 @@ class FlinkTasks(object):
 
         # copy caller details from context unless explictly set already
         if not task_request.HasField('reply_address') and not task_request.HasField('reply_topic'):
-            caller = context.get_caller()
 
-            if caller is not None:
-                task_request.reply_address.CopyFrom(caller)
+            address, caller_id = self._get_caller_address_and_id(context, task_request)
+            
+            if address is not None and caller_id is not None:
+                namespace, function_type = address.split('/')
+                task_request.reply_address.CopyFrom(Address(namespace=namespace, type=function_type, id=caller_id))
 
         return task_request
 
@@ -284,13 +284,15 @@ class FlinkTasks(object):
         args, kwargs, state = self._serialiser.deserialise_request(task_request)
         return args, kwargs, state
 
-    def send_result(self, context: TaskContext, task_request: TaskRequest, result, state=Ellipsis):
+    def send_result(self, context: TaskContext, task_request: TaskRequest, result, state=Ellipsis, delay: timedelta=None, cancellation_token: str = ""):
         """
         Sends a result
         :param context: TaskContext
         :param task_request: the incoming TaskRequest
         :param result: the result(s) to return
         :param state: the state to include in the result.  If not specified it will be copied from the TaskRequest
+        :param optional delay: the delay before Flink sends the result
+        :param optional cancellation_token: a cancellation token to associate with this message
         """
         task_result = _create_task_result(task_request)
 
@@ -299,21 +301,19 @@ class FlinkTasks(object):
 
         self._serialiser.serialise_result(task_result, result, state)
 
-        self.emit_result(context, task_request, task_result)
+        self.emit_result(context, task_request, task_result, delay,  cancellation_token)
 
-    def fail(self, context, task_input, ex):
+    def fail(self, context, task_input, ex, delay: timedelta=None, cancellation_token: str = ""):
         """
         Sends a failure
         :param context: TaskContext
         :param task_input: the incoming TaskRequest or TaskActionRequest
         :param ex: the exception to return
+        :param optional delay: the delay before Flink sends the result
+        :param optional cancellation_token: a cancellation token to associate with this message
         """
         task_exception = _create_task_exception(task_input, ex)
-
-        del context.storage.task_result
-        context.storage.task_exception = task_exception
-
-        self.emit_result(context, task_input, task_exception)
+        self.emit_result(context, task_input, task_exception, delay,  cancellation_token)
 
     def get_pipeline(self, context):
         pipeline_protos = context.pipeline_state.pipeline
@@ -329,8 +329,8 @@ class FlinkTasks(object):
         except:
             return None
 
-    def emit_result(self, context, task_input, task_result):
-        # set invocation id
+    def emit_result(self, context, task_input, task_result, delay: timedelta=None, cancellation_token: str = None):
+        # copy over invocation id
         if isinstance(task_result, (TaskResult, TaskException)):
             task_result.invocation_id = task_input.invocation_id
             
@@ -341,18 +341,22 @@ class FlinkTasks(object):
         # or call back to a particular flink function if reply_address was specified
         elif task_input.HasField('reply_address'):
             address, identifer = context.to_address_and_id(task_input.reply_address)
-            context.send_message(address, identifer, task_result)
+            context.send_message(address, identifer, task_result, delay, cancellation_token)
 
         elif isinstance(task_input, TaskRequest):
-            # if this was a task request it could be part of a pipeline - i.e. called from another tasks so
-            # either call back to original caller (e.g. if this is a result of a retry)
-            task_state = context.task_state.by_uid[task_input.uid]
-            if task_state.original_caller_id != '':
-                context.send_message(task_state.original_caller_address, task_state.original_caller_id, task_result)
+            address, caller_id = self._get_caller_address_and_id(context, task_input)
+            if address is not None and caller_id is not None:
+                context.send_message(address, caller_id, task_result, delay, cancellation_token)
 
-            # or call back to our caller (if there is one)
-            elif context.get_caller_id() is not None:
-                context.send_message(context.get_caller_address(), context.get_caller_id(), task_result)
-
-            # clean up
+        # clean up
+        if task_input.uid in context.task_state.by_uid:
             del context.task_state.by_uid[task_input.uid]
+
+    def _get_caller_address_and_id(self, context, task_request):
+        task_state = context.task_state.by_uid[task_request.uid]
+        
+        # either as original caller (e.g. if this is a result of a retry) or context.get_caller_...() otherwise
+        address = task_state.original_caller_address if task_state.original_caller_address != '' else context.get_caller_address()
+        caller_id = task_state.original_caller_id if task_state.original_caller_id != '' else context.get_caller_id()
+
+        return address, caller_id
