@@ -1,6 +1,7 @@
 from abc import ABC, abstractmethod
 from statefun_tasks.context import TaskContext
 from statefun_tasks.serialisation import DefaultSerialiser
+from statefun import Message
 
 
 class MessageHandler(ABC):
@@ -8,9 +9,9 @@ class MessageHandler(ABC):
         self._serialiser = serialiser
 
     @abstractmethod
-    def can_handle(self, context: TaskContext, message):
+    def unpack(self, context: TaskContext, message: Message):
         pass
 
     @abstractmethod
-    async def handle_message(self, tasks: 'FlinkTasks', context: TaskContext, message):
+    async def handle_message(self, tasks: 'FlinkTasks', context: TaskContext, unpacked_message):
         pass
